"""Module for some named entities."""

# Dictionaries with keywords for research areas:


def keywords():
    """Get named entities."""
    keys = {
        "solarMaterials": [
            "solar",
            "power conversion efficiency",
            "pce",
            "schottky-queisser limit",
            "short circuit current",
            "isc",
            "voc",
            "fill factor",
            "diffusion length",
            "transmission",
            "scatter",
            "photon",
            "renewable",
            "photovoltaic",
            "infrared",
            "lifetime",
            "diffusion coefficient",
            "impedance spectroscopy",
            "optical interference",
            "schottky",
            "junction",
            "ohmic",
            "heterojunction",
            "homojunction",
            "band alignment",
            "current density",
            "solar simulator",
            "spectral irradiance",
            "conductivity",
            "resistivity",
            "sheet resistance",
            "open circuit voltage",
            "absorption coefficient",
            "tandem solar cell",
            "drift",
            "band bending",
        ],
        "magneticMaterials": [
            "magnetic",
            "hysteresis",
            "curie",
            "magneton",
            "coercive field",
            "susceptibility",
        ],
        "batteryMaterials": [
            "cathode",
            "anode",
            "electrode",
            "electrolyte",
            "capacity",
            "energy density",
            "separator",
            "ionic conductivity",
            "electric vehicle",
            "battery",
            "energy storage",
            "nernst",
            "faraday",
            "faradaic charge transfer",
            "pseudocapacitor",
            "pseudocapacitance",
            "surface-limited",
            "electrochemistry",
            "electrochemical",
            "supercapacitor",
            "ion mobility",
            "diffusion coefficient",
            "ac impedance",
            "electrochemical impedance spectroscopy",
            "overpotential",
            "reduction potential",
            "oxidation potential",
            "redox",
            "current density",
            "cycle number",
            "cycle life",
            "discharge cycle",
            "cycle lifetime",
            "diffusion limited",
            "dendrite",
        ],
        "2DMaterials": [
            "interlayer excitions",
            "exciton binding energy",
            "flexible",
            "vdw",
            "exfoliate",
            "cvd",
            "mixed-dimensional heterostructure",
            "excitonics",
            "valleytronics",
            "flexible electronics",
            "tmd",
            "chalcogenide",
            "mxene",
            "xene",
            "mof",
            "perovskite",
            "cof",
            "spintronics",
            "cswap",
            "vth",
            "electrostatic doping",
            "defect passivation",
            "metal organic framework",
            "moire superlattice",
            "bilayer",
            "trilayer",
            "monolayer",
            "mechanical exfoliation",
            "trion",
            "van der waals",
            "wearable",
            "graphene",
            "threshold voltage",
            "covalent organic framework",
            "polariton",
            "valley degree of freedom",
        ],
        "photocatalystMaterials": [
            "charge transfer",
            "inorganic coordination complex",
            "transition metal",
            "co2 reduction",
            "water splitting",
            "n2 fixation",
            "charge separation",
        ],
    }
    return keys
